searchData={"items":[{"type":"module","title":"Replika","doc":"Replika is a pure functional finite state machine library for Elixir.\n\nUnlike process-based state machines, Replika implements\nstate machines as immutable data structures that transform as they\ntransition between states. This provides significant performance\nbenefits while simplifying state management.","ref":"Replika.html"},{"type":"module","title":"Key Benefits - Replika","doc":"* **High Performance** - State transitions are simple struct updates (O(1))\n* **No Process Overhead** - Works as a lightweight data structure\n* **Immutable & Pure** - Predictable state transitions without side effects\n* **Embeddable** - Can be stored in ETS, databases, or embedded in other processes\n* **Pattern Matching** - Pattern matching is optimized by the BEAM VM\n* **Deterministic Memory Usage** - Low memory footprint at scale","ref":"Replika.html#module-key-benefits"},{"type":"module","title":"Structure - Replika","doc":"When you use Replika in a module, it defines a struct with the following fields:\n\n```elixir\n%YourModule{\n  state: atom(),  # The current state of the FSM\n  data: any()     # The data associated with the current state\n}\n```\n\nThe `state` field holds the current state name (atom), and the `data`\nfield can hold any associated data for that state.","ref":"Replika.html#module-structure"},{"type":"module","title":"Example - Replika","doc":"The following diagram illustrates a simple state machine with four states:\n\n```\n        +---------------+\n        |               v\n+----------------+    +----------------+    +-------------------+\n| :no_id_card    |--->| :has_id_card   |--->| :at_elevator      |\n+----------------+    +----------------+    +-------------------+\n                                                   |\n                                                   v\n                                           +--------------------+\n                                           | :elevator_accessed |\n                                           +--------------------+\n```\n\nThis diagram can then be represented by the following:\n\n```elixir\ndefmodule Unit.AccessState do\n  use Replika, initial_state: :no_id_card, initial_data: %{inventory: []}\n\n  defstate no_id_card do\n    defevent pickup_card do\n      next_state(:has_id_card, %{inventory: [:id_card]})\n    end\n  end\n\n  defstate has_id_card do\n    defevent approach_elevator do\n      next_state(:at_elevator, %{inventory: [:id_card]})\n    end\n  end\n\n  defstate at_elevator do\n    defevent use_card, data: %{inventory: inventory} do\n      if :id_card in inventory do\n        next_state(:elevator_accessed, %{inventory: inventory})\n      else\n        # Remain in the same state if missing card\n        next_state(:at_elevator, %{inventory: inventory})\n      end\n    end\n\n    defevent leave_elevator do\n      next_state(:has_id_card, %{inventory: [:id_card]})\n    end\n  end\n\n  defstate elevator_accessed do\n    defevent select_floor(floor) do\n      respond({:travelling_to, floor}, :elevator_accessed, %{inventory: [:id_card]})\n    end\n  end\nend\n```","ref":"Replika.html#module-example"},{"type":"module","title":"Usage: - Replika","doc":"We can interact with the created state machine like this:\n\n```elixir\nelster = Unit.AccessState.new()\n\n# Progress through the game\nelster = elster\n           |> Unit.AccessState.pickup_card()\n           |> Unit.AccessState.approach_elevator()\n           |> Unit.AccessState.use_card()\n\n# Unit can now use the elevator\nUnit.AccessState.state(elster)  # :elevator_accessed\n\n# Select a floor\n{response, elster} = Unit.AccessState.select_floor(elster, \"b6\")\n```","ref":"Replika.html#module-usage"},{"type":"module","title":"Working with Data - Replika","doc":"FSMs often need to carry data alongside their state:\n\n```elixir\ndefmodule InventorySystem do\n  use Replika, initial_state: :empty, initial_data: %{items: []}\n\n  defstate empty do\n    defevent add_item(item), data: %{items: items} do\n      new_items = [item | items]\n      if length(new_items) > 0 do\n        next_state(:has_items, %{items: new_items})\n      else\n        next_state(:empty, %{items: new_items})\n      end\n    end\n  end\n\n  defstate has_items do\n    defevent add_item(item), data: %{items: items} do\n      next_state(:has_items, %{items: [item | items]})\n    end\n\n    defevent remove_item(item), data: %{items: items} do\n      new_items = List.delete(items, item)\n      if new_items == [] do\n        next_state(:empty, %{items: []})\n      else\n        next_state(:has_items, %{items: new_items})\n      end\n    end\n\n    defevent list_items, data: %{items: items} do\n      respond(items)\n    end\n  end\nend\n```","ref":"Replika.html#module-working-with-data"},{"type":"module","title":"Error Handling - Replika","doc":"Replika provides clear error messages for invalid transitions:\n\n```elixir\n# Trying an invalid transition\ninventory = InventorySystem.new()\n           |> InventorySystem.add_item(\"Keycard\")\n\n# This would cause an error because 'list_items' is only defined in :has_items state\nInventorySystem.list_items(inventory)\n\n# You'll see a clear error:\n# ** (Replika.Error.InvalidTransitionError) Invalid transition: cannot execute\n#    event 'list_items' with args [] in state ':empty'\n```","ref":"Replika.html#module-error-handling"},{"type":"module","title":"Pattern Matching and Guards - Replika","doc":"You can leverage Elixir's pattern matching and guards for sophisticated state logic:\n\n```elixir\ndefstate at_elevator do\n  # Different handling based on item in inventory\n  defevent use_card, data: %{inventory: inventory} when :id_card in inventory do\n    next_state(:elevator_accessed, %{inventory: inventory})\n  end\n\n  # Handle case where ID card is missing\n  defevent use_card, data: %{inventory: inventory} do\n    respond({:error, :missing_id_card}, :at_elevator, %{inventory: inventory})\n  end\nend\n```","ref":"Replika.html#module-pattern-matching-and-guards"},{"type":"macro","title":"Replika.__using__/1","doc":"When used, defines a new Replika state machine.","ref":"Replika.html#__using__/1"},{"type":"macro","title":"Options - Replika.__using__/1","doc":"* `:initial_state` - The initial state of the FSM (required)\n* `:initial_data` - The initial data of the FSM (optional, defaults to `nil`)","ref":"Replika.html#__using__/1-options"},{"type":"macro","title":"Examples - Replika.__using__/1","doc":"defmodule Unit.AccessState do\n      use Replika, initial_state: :no_id_card, initial_data: %{inventory: []}\n    end","ref":"Replika.html#__using__/1-examples"},{"type":"type","title":"Replika.action_response/0","doc":"Action responses returned by event handlers.\n\n  Can be one of:\n  - `{:next_state, atom()}` - Change state\n  - `{:new_data, any()}` - Update data\n  - `{:respond, any()}` - Return a value","ref":"Replika.html#t:action_response/0"},{"type":"macro","title":"Replika.defevent/1","doc":"Declares an event in the FSM without implementation.","ref":"Replika.html#defevent/1"},{"type":"macro","title":"Parameters - Replika.defevent/1","doc":"- `event`: The name of the event (and optionally its arity)","ref":"Replika.html#defevent/1-parameters"},{"type":"macro","title":"Examples - Replika.defevent/1","doc":"# Declare a zero-arity event\n    defevent pickup_card\n\n    # Declare a two-arity event\n    defevent use_item/2","ref":"Replika.html#defevent/1-examples"},{"type":"macro","title":"Replika.defevent/2","doc":"Defines an event in the FSM with options but no implementation block.","ref":"Replika.html#defevent/2"},{"type":"macro","title":"Parameters - Replika.defevent/2","doc":"- `event`: The name of the event\n- `opts`: Options for the event (must include :do option with the implementation)","ref":"Replika.html#defevent/2-parameters"},{"type":"macro","title":"Examples - Replika.defevent/2","doc":"# Event with implementation in the options\n    defevent pickup_card, do: next_state(:has_id_card, %{inventory: [:id_card]})","ref":"Replika.html#defevent/2-examples"},{"type":"macro","title":"Replika.defevent/3","doc":"Defines an event in the FSM with options and implementation block.","ref":"Replika.html#defevent/3"},{"type":"macro","title":"Parameters - Replika.defevent/3","doc":"- `event`: The name of the event\n- `opts`: Options for the event\n- `event_def`: The event implementation block","ref":"Replika.html#defevent/3-parameters"},{"type":"macro","title":"Examples - Replika.defevent/3","doc":"# Event with options and block\n    defevent use_card, data: %{inventory: inventory} do\n      if :id_card in inventory do\n        next_state(:elevator_accessed, %{inventory: inventory})\n      else\n        next_state(:at_elevator, %{inventory: inventory})\n      end\n    end","ref":"Replika.html#defevent/3-examples"},{"type":"macro","title":"Replika.defeventp/1","doc":"Declares a private event in the FSM without implementation.\n\nWorks the same as `defevent/1` but generates a private function instead of a public one.","ref":"Replika.html#defeventp/1"},{"type":"macro","title":"Examples - Replika.defeventp/1","doc":"# Declare a private zero-arity event\n    defeventp internal_transition","ref":"Replika.html#defeventp/1-examples"},{"type":"macro","title":"Replika.defeventp/2","doc":"Defines a private event in the FSM with options but no implementation block.","ref":"Replika.html#defeventp/2"},{"type":"macro","title":"Parameters - Replika.defeventp/2","doc":"- `event`: The name of the event\n- `opts`: Options for the event (must include :do option with the implementation)","ref":"Replika.html#defeventp/2-parameters"},{"type":"macro","title":"Examples - Replika.defeventp/2","doc":"# Private event with implementation in the options\n    defeventp internal_scan, do: next_state(:scanning)","ref":"Replika.html#defeventp/2-examples"},{"type":"macro","title":"Replika.defeventp/3","doc":"Defines a private event in the FSM with options and implementation block.","ref":"Replika.html#defeventp/3"},{"type":"macro","title":"Parameters - Replika.defeventp/3","doc":"- `event`: The name of the event\n- `opts`: Options for the event\n- `event_def`: The event implementation block","ref":"Replika.html#defeventp/3-parameters"},{"type":"macro","title":"Examples - Replika.defeventp/3","doc":"# Private event with options and block\n    defeventp validate_card, data: %{inventory: inventory} do\n      :id_card in inventory\n    end","ref":"Replika.html#defeventp/3-examples"},{"type":"macro","title":"Replika.defstate/2","doc":"Defines a state in the FSM.","ref":"Replika.html#defstate/2"},{"type":"macro","title":"Parameters - Replika.defstate/2","doc":"- `state`: The name of the state\n- `state_def`: The state definition block","ref":"Replika.html#defstate/2-parameters"},{"type":"macro","title":"Examples - Replika.defstate/2","doc":"defstate at_elevator do\n      defevent use_card, data: %{inventory: inventory} do\n        if :id_card in inventory do\n          next_state(:elevator_accessed, %{inventory: inventory})\n        else\n          next_state(:at_elevator, %{inventory: inventory})\n        end\n      end\n    end","ref":"Replika.html#defstate/2-examples"},{"type":"function","title":"Replika.next_state/1","doc":"Transitions to a new state without changing data.\n\nThis function is used within event handlers to change the state machine's\nstate while preserving its existing data.","ref":"Replika.html#next_state/1"},{"type":"function","title":"Parameters - Replika.next_state/1","doc":"- `state`: The target state to transition to (atom)","ref":"Replika.html#next_state/1-parameters"},{"type":"function","title":"Returns - Replika.next_state/1","doc":"An action response tuple `{:action_responses, [{:next_state, atom()}]}`\nthat will be processed by the state machine.","ref":"Replika.html#next_state/1-returns"},{"type":"function","title":"Examples - Replika.next_state/1","doc":"defevent pickup_card do\n      next_state(:has_id_card)  # Transition to :has_id_card state\n    end","ref":"Replika.html#next_state/1-examples"},{"type":"function","title":"Replika.next_state/2","doc":"Transitions to a new state and updates data.\n\nThis function is used within event handlers to simultaneously change the\nstate machine's state and update its associated data.","ref":"Replika.html#next_state/2"},{"type":"function","title":"Parameters - Replika.next_state/2","doc":"- `state`: The target state to transition to (atom)\n- `data`: The new data value to store","ref":"Replika.html#next_state/2-parameters"},{"type":"function","title":"Returns - Replika.next_state/2","doc":"An action response tuple `{:action_responses, [{:next_state, atom()}, {:new_data, any()}]}`\nthat will be processed by the state machine.","ref":"Replika.html#next_state/2-returns"},{"type":"function","title":"Examples - Replika.next_state/2","doc":"defevent pickup_card do\n      next_state(:has_id_card, %{inventory: [:id_card]})\n    end","ref":"Replika.html#next_state/2-examples"},{"type":"function","title":"Replika.respond/1","doc":"Returns a response without changing state or data.\n\nThis function is used within event handlers to return a value to the caller\nwithout modifying the state machine's state or data.","ref":"Replika.html#respond/1"},{"type":"function","title":"Parameters - Replika.respond/1","doc":"- `response`: The value to return from the event handler","ref":"Replika.html#respond/1-parameters"},{"type":"function","title":"Examples - Replika.respond/1","doc":"defevent list_items, data: %{items: items} do\n      respond(items)  # Return items to caller, no state/data change\n    end","ref":"Replika.html#respond/1-examples"},{"type":"function","title":"Replika.respond/2","doc":"Returns a response and transitions to a new state.","ref":"Replika.html#respond/2"},{"type":"function","title":"Parameters - Replika.respond/2","doc":"- `response`: The value to return from the event handler\n- `state`: The state to transition to","ref":"Replika.html#respond/2-parameters"},{"type":"function","title":"Examples - Replika.respond/2","doc":"defevent examine_card do\n      respond({:card_info, \"SECTOR B ACCESS\"}, :has_id_card)\n    end","ref":"Replika.html#respond/2-examples"},{"type":"function","title":"Replika.respond/3","doc":"Returns a response, transitions to a new state, and updates data.","ref":"Replika.html#respond/3"},{"type":"function","title":"Parameters - Replika.respond/3","doc":"- `response`: The value to return from the event handler\n- `state`: The state to transition to\n- `data`: The new data value","ref":"Replika.html#respond/3-parameters"},{"type":"function","title":"Examples - Replika.respond/3","doc":"defevent select_floor(floor) do\n      respond({:travelling_to, floor}, :elevator_accessed, %{inventory: [:id_card]})\n    end","ref":"Replika.html#respond/3-examples"},{"type":"module","title":"Replika.Error","doc":"Error types for Replika state machines.\n\nThese errors provide detailed diagnostic information to help\ntroubleshoot issues with state machine definitions and usage.","ref":"Replika.Error.html"},{"type":"exception","title":"Replika.Error.InvalidTransitionError","doc":"Raised when attempting an event transition that isn't defined for the current state.\n\nThis error provides detailed information about:\n- The current state of the FSM\n- The event that was attempted\n- Any arguments passed to the event","ref":"Replika.Error.InvalidTransitionError.html"},{"type":"exception","title":"Example - Replika.Error.InvalidTransitionError","doc":"try do\n      Unit.AccessState.new()\n      |> Unit.AccessState.pickup_card()\n      |> Unit.AccessState.pickup_card()  # Can't pickup card when already have one\n    rescue\n      e in InvalidTransitionError ->\n        # e.state == :has_id_card\n        # e.event == :pickup_card\n        # e.args == []\n        IO.puts(e.message)\n    end","ref":"Replika.Error.InvalidTransitionError.html#module-example"},{"type":"exception","title":"Troubleshooting - Replika.Error.InvalidTransitionError","doc":"This error usually indicates one of these issues:\n1. Calling an event in the wrong state (state machine logic error)\n2. Missing a state definition for a necessary transition\n3. Missing an event definition for a valid use case\n\nReview your state machine definition and ensure all valid state transitions\nare properly defined.","ref":"Replika.Error.InvalidTransitionError.html#module-troubleshooting"},{"type":"type","title":"Replika.Error.InvalidTransitionError.t/0","doc":"","ref":"Replika.Error.InvalidTransitionError.html#t:t/0"},{"type":"extras","title":"Replika 🦾","doc":"# Replika 🦾\n\nA pure functional finite state machine library for Elixir.","ref":"readme.html"},{"type":"extras","title":"Table of Contents 📖 - Replika 🦾","doc":"- [Overview](#overview)\n- [Installation](#installation)\n- [Usage](#usage)\n- [Error Handling](#error-handling)\n- [Pattern Matching and Guards](#pattern-matching-and-guards)\n- [Returning Values](#returning-values)\n- [Global Event Handlers](#global-event-handlers)\n- [Dynamic FSM Creation](#dynamic-fsm-creation)\n- [Performance](#performance)\n- [Advanced Usage](#advanced-usage)\n- [Testing Replika State Machines](#testing-replika-state-machines)\n- [Documentation](#documentation)\n- [License](#license)","ref":"readme.html#table-of-contents"},{"type":"extras","title":"Overview 🔍 - Replika 🦾","doc":"Replika implements finite state machines as immutable data structures that transform during state transitions. This approach provides:\n\n* **High Performance** - State transitions are simple struct updates (O(1))\n* **No Process Overhead** - Works as a lightweight data structure\n* **Immutable & Pure** - Predictable state transitions without side effects\n* **Embeddable** - Can be stored in ETS, databases, or embedded in other processes\n* **Pattern Matching** - Pattern matching is optimized by the BEAM VM\n* **Deterministic Memory Usage** - Low memory footprint at scale","ref":"readme.html#overview"},{"type":"extras","title":"Installation 📦 - Replika 🦾","doc":"Add Replika to your `mix.exs` dependencies:\n\n```elixir\ndef deps do\n  [\n    {:replika, \"~> 0.1.1\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Usage 🧑‍🏫 - Replika 🦾","doc":"This example shows a state machine for a character accessing secured areas. The character starts with no ID card, can pick one up, approach an elevator, and use the card to gain access.\n\nEach state defines valid transitions to other states, and each transition can include data transformation:\n\n```elixir\ndefmodule Unit.AccessState do\n  use Replika, initial_state: :no_id_card, initial_data: %{inventory: []}\n\n  defstate no_id_card do\n    defevent pickup_card do\n      next_state(:has_id_card, %{inventory: [:id_card], floors_visited: []})\n    end\n  end\n\n  defstate has_id_card do\n    defevent approach_elevator do\n      next_state(:at_elevator, %{inventory: [:id_card], floors_visited: []})\n    end\n  end\n\n  defstate at_elevator do\n    defevent use_card, data: %{inventory: inventory, floors_visited: visited} do\n      if :id_card in inventory do\n        next_state(:elevator_accessed, %{inventory: inventory, floors_visited: visited})\n      else\n        next_state(:at_elevator, %{inventory: inventory, floors_visited: visited})\n      end\n    end\n  end\n\n  defstate elevator_accessed do\n    defevent select_floor(floor), data: %{inventory: inv, floors_visited: visited} do\n      next_state(:elevator_accessed, %{inventory: inv, floors_visited: [floor | visited]})\n    end\n\n    defevent get_visited_floors, data: %{floors_visited: visited} do\n      respond(visited)\n    end\n  end\nend\n```\n\nWe can then interact with the state machine like this:\n\n```elixir\n\n# Create a new FSM instance\nelster = Unit.AccessState.new()\n\n# Progress through states by calling event functions\nelster = elster\n         |> Unit.AccessState.pickup_card()\n         |> Unit.AccessState.approach_elevator()\n         |> Unit.AccessState.use_card()\n\n# Inspect the current state\nUnit.AccessState.state(elster)  # Returns :elevator_accessed\n\n# Events can return values while changing state\n{floors, elster} = Unit.AccessState.get_visited_floors(elster)\n```\n\nThe state machine above defines:\n\n* Four distinct states: `:no_id_card`, `:has_id_card`, `:at_elevator`, and `:elevator_accessed`\n* Events that trigger transitions between states\n* Data transformations with each state change\n* Conditional transitions based on inventory contents\n\nEach event function returns either the updated FSM instance or a tuple containing a response value AND the updated instance.","ref":"readme.html#usage"},{"type":"extras","title":"Error Handling ⚠️ - Replika 🦾","doc":"Replika provides clear error messages when you attempt invalid transitions. If you try to execute an event that isn't defined for the current state, you'll get a helpful error message:\n\n```elixir\ntry do\n  # Starting with no card\n  elster = Unit.AccessState.new()\n\n  # Pick up a card, putting us in :has_id_card state\n  elster = Unit.AccessState.pickup_card(elster)\n\n  # Try to pick up a card again - but we're already in :has_id_card state!\n  # This raises an InvalidTransitionError since pickup_card is only defined\n  # for the :no_id_card state\n  Unit.AccessState.pickup_card(elster)\nrescue\n  e in Replika.Error.InvalidTransitionError ->\n    IO.puts(e.message)\n    # \"Invalid transition: cannot execute event 'pickup_card' with args [] in state ':has_id_card'\"\nend\n```\n\nThe error message includes:\n\n* The current state (`:has_id_card`)\n* The invalid event (`pickup_card`)\n* Any arguments passed to the event (`[]`)\n\nThis makes debugging much easier compared to normal function clause errors.","ref":"readme.html#error-handling"},{"type":"extras","title":"Pattern Matching and Guards 🧩 - Replika 🦾","doc":"Pattern matching lets you handle different cases within the same state based on data conditions. Combined with guards, this provides powerful control flow:\n\n```elixir\ndefstate at_elevator do\n  # First clause: match when ID card is in inventory\n  # The guard clause (when) ensures this only matches when :id_card is in inventory\n  defevent use_card, data: %{inventory: inventory} when :id_card in inventory do\n    next_state(:elevator_accessed, %{inventory: inventory})\n  end\n\n  # Second clause: match when ID card is NOT in inventory (fallback case)\n  # This handles the case where inventory doesn't contain :id_card\n  defevent use_card, data: %{inventory: inventory} do\n    # Return an error response and remain in the same state\n    respond({:error, :missing_id_card}, :at_elevator, %{inventory: inventory})\n  end\nend\n```\n\nThis pattern lets you:\n\n* Define different behaviors for the same event based on data conditions\n* Use guard clauses for precise control over which clause matches\n* Return different values or transition to different states based on conditions\n* Handle error cases elegantly","ref":"readme.html#pattern-matching-and-guards"},{"type":"extras","title":"Returning Values 📤 - Replika 🦾","doc":"Events can return values to the caller while optionally changing state. This is useful for querying the FSM or implementing commands that produce a result:\n\n```elixir\ndefstate elevator_accessed do\n  # Query event: returns data without changing state\n  defevent get_visited_floors, data: %{floors_visited: visited} do\n    # First arg is the return value, no state change\n    respond(visited)\n  end\n\n  # Command event: returns data AND changes state\n  defevent select_floor(floor), data: %{inventory: inv, floors_visited: visited} do\n    # Return a tuple, update state, and modify data\n    respond(\n      {:travelling_to, floor},                             # Return value\n      :elevator_accessed,                                  # New state (unchanged)\n      %{inventory: inv, floors_visited: [floor | visited]} # New data\n    )\n  end\nend\n```\n\nUsing `respond/1`, `respond/2`, or `respond/3`:\n\n* `respond(value)` - Returns value without changing state or data\n* `respond(value, new_state)` - Returns value and changes state\n* `respond(value, new_state, new_data)` - Returns value, changes state, and updates data\n\nWhen using respond, the event call returns a tuple: `{return_value, updated_fsm}`.","ref":"readme.html#returning-values"},{"type":"extras","title":"Global Event Handlers 🌐 - Replika 🦾","doc":"Sometimes you need to handle unexpected events or provide fallback behavior. The special `_` event can catch undefined events:\n\n```elixir\ndefmodule SecuritySystem do\n  use Replika, initial_state: :armed, initial_data: %{alert_count: 0}\n\n  defstate armed do\n    # Defined events\n    defevent disarm(code) when code == 1234 do\n      next_state(:disarmed)\n    end\n\n    # Catch-all for any other event in the :armed state\n    # This will catch any undefined event while in the armed state\n    defevent _ do\n      # Increment alert count and remain armed\n      next_state(:armed, %{alert_count: data.alert_count + 1})\n    end\n  end\n\n  defstate disarmed do\n    defevent arm do\n      next_state(:armed, %{alert_count: 0})\n    end\n  end\n\n  # Global catch-all for any undefined event in any state\n  # This only triggers if there's no state-specific handler\n  defevent _ do\n    respond({:error, :invalid_operation})\n  end\nend\n```\n\nThe event handlers are checked in order:\n\n1. Exact event/state match\n2. State-specific catch-all (`_` event in the current state)\n3. Global catch-all (`_` event outside any state)\n\nThis gives you complete control over error handling and fallback behavior.","ref":"readme.html#global-event-handlers"},{"type":"extras","title":"Dynamic FSM Creation 🏗️ - Replika 🦾","doc":"Replika's macros enable programmatic FSM definition - useful for state machines with regular patterns:\n\n```elixir\ndefmodule ProtektorElevator do\n  use Replika, initial_state: :b1, initial_data: %{log: []}\n\n  # Define all possible floors\n  floors = [:b1, :b2, :b3, :b4, :b5, :b6]\n\n  # Generate states and transitions dynamically\n  for current_floor <- floors do\n    defstate current_floor do\n      # For each floor, create transitions to all other floors\n      for target_floor <- floors, target_floor != current_floor do\n        # Create a \"go_to_X\" event for each possible target floor\n        defevent :\"go_to_#{target_floor}\", data: %{log: log} do\n          # Log the transition and update state\n          next_state(\n            target_floor,\n            %{log: [\"#{current_floor} -> #{target_floor}\" | log]}\n          )\n        end\n      end\n\n      # Add inspection event to see travel history\n      defevent :travel_history, data: %{log: log} do\n        respond(log)\n      end\n    end\n  end\nend\n```\n\nThis generates a complete elevator control system with:\n\n* One state for each floor\n* Events to travel between any floors\n* Event naming that matches the destination (e.g., `go_to_b6`)\n* Automatic logging of all floor transitions\n\nUsing metaprogramming this way creates sophisticated state machines with minimal code.","ref":"readme.html#dynamic-fsm-creation"},{"type":"extras","title":"Performance ⏱️ - Replika 🦾","doc":"Replika FSMs are lightweight data structures that make state transitions in constant time. There's no process creation, message passing, or serialization overhead. This makes Replika well-suited for high-throughput and memory-constrained applications.\n\nRun the benchmarks to compare with `gen_state_machine`, an Elixir wrapper for Erlang's `gen_statem`:\n\n```\nmix bench\n```\n\nor\n\n```\nmix run bench/replika_bench.exs\n```","ref":"readme.html#performance"},{"type":"extras","title":"Advanced Usage 🔧 - Replika 🦾","doc":"While Replika provides exceptional performance as a pure functional state machine, you can combine it with OTP's process model when you need features like supervision or timeouts.\n\n```elixir\ndefmodule Protektor.ElevatorServer do\n  use GenServer\n\n  def start_link(args), do: GenServer.start_link(__MODULE__, args)\n\n  def init(args), do: {:ok, Unit.AccessState.new(args)}\n\n  # Expose FSM events as server API\n  def pickup_card(pid), do: GenServer.call(pid, :pickup_card)\n  def use_card(pid), do: GenServer.call(pid, :use_card)\n\n  # Handle the events in the server\n  def handle_call(:pickup_card, _from, fsm) do\n    new_fsm = Unit.AccessState.pickup_card(fsm)\n    {:reply, :ok, new_fsm}\n  end\n\n  def handle_call(:use_card, _from, fsm) do\n    new_fsm = Unit.AccessState.use_card(fsm)\n    {:reply, :ok, new_fsm}\n  end\n\n  # Timeout example\n  def handle_info(:security_timeout, fsm) do\n    IO.puts(\"ALERT: Security timeout - initiating lockdown\")\n    {:noreply, fsm}\n  end\nend\n```\n\nThis hybrid approach allows you to take advantage of:\n\n* OTP Integration - Supervision trees and fault tolerance\n* Distribution - Run FSMs across a cluster\n* Timeouts - Support for time-based transitions and alerts\n* Long-Running Operations - Background tasks and periodic checks\n\nIn addition, you still get the performance benefits of Replika's faster transitions, lower resource use, and simpler state logic.\n\nUse this pattern when you need both the reliability of OTP and the performance of Replika.","ref":"readme.html#advanced-usage"},{"type":"extras","title":"Testing Replika State Machines 🧪 - Replika 🦾","doc":"Testing Replika state machines is straightforward since they're just data structures:\n\n```elixir\ndefmodule Unit.AccessStateTest do\n  use ExUnit.Case\n\n  test \"unit can access elevator with ID card\" do\n    # Start with no ID card\n    unit = Unit.AccessState.new()\n    assert Unit.AccessState.state(unit) == :no_id_card\n\n    # Pick up ID card and approach elevator\n    unit = unit\n           |> Unit.AccessState.pickup_card()\n           |> Unit.AccessState.approach_elevator()\n\n    assert Unit.AccessState.state(unit) == :at_elevator\n\n    # Use card to access elevator\n    unit = Unit.AccessState.use_card(unit)\n    assert Unit.AccessState.state(unit) == :elevator_accessed\n\n    # Check visited floors\n    {floors, _unit} = Unit.AccessState.get_visited_floors(unit)\n    assert floors == []\n\n    # Select a floor\n    {response, unit} = Unit.AccessState.select_floor(unit, \"b6\")\n    assert response == {:travelling_to, \"b6\"}\n\n    # Verify floor was added to visited floors\n    {floors, _unit} = Unit.AccessState.get_visited_floors(unit)\n    assert \"b6\" in floors\n  end\n\n  test \"raises error for invalid transitions\" do\n    unit = Unit.AccessState.new()\n\n    assert_raise Replika.Error.InvalidTransitionError, fn ->\n      # Can't use card without approaching elevator first\n      Unit.AccessState.use_card(unit)\n    end\n\n    unit = Unit.AccessState.pickup_card(unit)\n\n    assert_raise Replika.Error.InvalidTransitionError, fn ->\n      # Can't pick up card when already have one\n      Unit.AccessState.pickup_card(unit)\n    end\n  end\nend\n```\n\nSince Replika state machines are deterministic and free from side-effects, they're easy to test without mocks or complex setups. You can create instances, apply transitions, and verify both state changes and returned values directly.","ref":"readme.html#testing-replika-state-machines"},{"type":"extras","title":"Documentation 📚 - Replika 🦾","doc":"For detailed API documentation, go to:\n\n- [Replika](https://hexdocs.pm/replika/Replika.html) - Main module documentation\n- [Replika.Error](https://hexdocs.pm/replika/Replika.Error.html) - Error handling","ref":"readme.html#documentation"},{"type":"extras","title":"License 📄 - Replika 🦾","doc":"This project is licensed under:\n\n[![License: MPL 2.0](https://img.shields.io/badge/License-MPL%202.0-brightgreen.svg)](https://www.mozilla.org/en-US/MPL/2.0/)","ref":"readme.html#license"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}